@@ -206,7 +206,7 @@ class RpcExchangerClient:
     async def send(self, msg: str, chat: int, session: str | None) -> None:
         self.check_authorized()
         self.check_joined(chat)
-        if session not in self._session_list:
+        if session is not None and (chat, session) not in self._session_list:
             raise ChatNotJoinedError()
         if not await self._rpc.chat.check_send(chat_id=chat, user_id=self._uid):
             raise PermissionDeniedError()
@@ -228,14 +228,14 @@ class RpcExchangerClient:
                 if raw_message["channel"].startswith(b"messages"):
                     json_message: RedisMessage = json_content_untyped
                     session: str | None = None
-                    if "session" in json_message:
-                        session = json_message["session"]
-                        if session not in self._session_list:
-                            continue
                     username = json_message["username"]
                     msg = json_message["msg"]
                     chat = int(raw_message["channel"][9:])
-                    log.debug(f"{username=} {msg=} {chat=}")
+                    if "session" in json_message:
+                        session = json_message["session"]
+                        if (chat, session) not in self._session_list:
+                            continue
+                    log.debug(f"{username=} {msg=} {chat=} {session=}")
                     return "message", username, msg, chat, session
                 elif raw_message["channel"].startswith(b"events"):
                     json_content: RedisEvent = json_content_untyped
@@ -306,7 +306,7 @@ class RpcExchangerClient:
             await self._pubsub.unsubscribe(f"messages:{id}")
             await self._pubsub.unsubscribe(f"events:{id}")
         async with self._chat_list_lock:
-            self._chat_list.remove(id)
+            self._chat_list.discard(id)
         return True
 
     async def chat_list_somebody_joined(self) -> list[tuple[int, str, int | None]]:
